const gulp = require('gulp')
const nodemon = require('gulp-nodemon')
const eslint = require('gulp-eslint')

const jsFiles = ['*.js', 'src/**/*.js', '*.json', 'src/**/*.json', '*.yaml', 'src/**/*.yaml']
const lintFiles = ['**/*.js', '!node_modules/**', '!public/js/vendor/**']

// ESLint ignores files with "node_modules" paths.
// So, it's best to have gulp ignore the directory as well.
// Also, Be sure to return the stream from the task;
// Otherwise, the task may end before the stream has finished.

// eslint() attaches the lint output to the "eslint" property
// of the file object so it can be used by other modules.

// eslint.format() outputs the lint results to the console.
// Alternatively use eslint.formatEach() (see Docs).

gulp.task('lint', () => gulp.src(lintFiles)
    .pipe(eslint({
        fix: true
    }))
    .pipe(eslint.format()))

gulp.task('watchLint', () => gulp.watch(lintFiles, {debounceDelay: 200}, gulp.series('lint')))

gulp.task('watchCode', () => nodemon({
    script: 'index.js',
    ext: 'js json',
    delay: 200,
    env: {
        PORT: 3001
    },
    watch: jsFiles
})
    .on('restart', () => {
        console.log('Restarting....')
    }))

gulp.task('serve', gulp.parallel('watchCode', 'watchLint'))
