const express = require('express')
const controller = require('./serumsController')()
const validateRequest = require('../../middleware/validateRequest')

const serumsRouter = express.Router()

const router = function router () {
  // Middleware
  validateRequest(serumsRouter, {
    publicUrls: [
      { url: '/ping', method: 'GET' },
      { url: '/reset', method: 'GET' }
    ],
    basicAuthorization: [
      { url: '/patientAccess', method: 'POST' },
      { url: '/accessLog', method: 'POST' },
      { url: '/proof', method: 'POST' },
      { url: '/proof', method: 'GET' },
      { url: '/proof', method: 'PATCH' }
    ]
  })

  serumsRouter.route('/rule').post(controller.createRule)
  serumsRouter.route('/rule/:ruleId').get(controller.getOneRule)
  serumsRouter.route('/rule/:ruleId').put(controller.updateRule)
  serumsRouter.route('/rule/:ruleId').delete(controller.deleteRule)
  serumsRouter.route('/getRules').post(controller.getRules)
  serumsRouter.route('/patientAccess').post(controller.getPatientAccess)
  serumsRouter.route('/accessLog').post(controller.getAccessLog)

  serumsRouter.route('/ping').get(controller.pingRequest)
  serumsRouter.route('/reset').get(controller.resetRequest)

  serumsRouter.route('/proof').post(controller.createProof)
  serumsRouter.route('/proof/:proofId').get(controller.getProof)
  serumsRouter.route('/proof/:proofId').patch(controller.updateProof)

  return serumsRouter
}

module.exports = router
