const request = require('request')
const jwt = require('jsonwebtoken')

const logger = require('../../utils/logging/logger').getLogger('services/authentication')

// Temporary solution until Auth module has correctly implemented SSL
process.env.NODE_TLS_REJECT_UNAUTHORIZED = 0

const authConfig = require('../../config/config').auth

// This function will validate the JWT token that has been sent in
// JWT token is received from the Authentication module made by UCY
// When we receive it, the only thing we have to do is validate it by using their endpoint
// We will then decode the token and use it in further requests
async function validate (jwtToken) {
  logger.info(`Validating authentication access token from ${authConfig.url}`)
  const options = {
    uri: `${authConfig.url}/verify_jwt/`,
    headers: { Authorization: `Bearer ${jwtToken}` }
  }
  return new Promise((resolve, reject) => {
    request.post(options, (error, response, body) => {
      if (error) return reject(error)
      try {
        return resolve({ ...JSON.parse(body), status: response.statusCode })
      } catch (err) {
        return reject(err)
      }
    })
  })
}

async function validateStaticJWT (authorization) {
  const token = authorization.split(' ')[1]

  return new Promise((resolve) => {
    jwt.verify(token, process.env.SECRET_KEY, (err) => {
      if (err) {
        return resolve(false)
      }
      return resolve(true)
    })
  })
}

/* eslint-disable */
function authenticationService (config) {
  // authConfig = config
  return {
    validateJWT: validate,
    validateStaticJWT
  }
}
/* eslint-enable */


module.exports = authenticationService
