const path = require('path')
const fs = require('fs')
// const FabricCAServices = require('fabric-ca-client')
const {FileSystemWallet, X509WalletMixin/* , Identity */} = require('fabric-network')
const {promisify} = require('util')

const networkConfig = require('../../config/network')
// const {IOrganizationConfig, ICAConfig} = require('../config/typings')

const readFile = promisify(fs.readFile)

const CA_ADMIN_LABEL = 'ca-admin'
const ADMIN_LABEL = 'admin'

class IdentityService {
    constructor(
        orgName, // string,
        client // FabricClient
    ) {
        this.orgName = orgName

        this.client = client

        // Create a new file system based wallet for managing identities.
        this.walletPath = path.resolve(process.cwd(), 'wallet', this.orgName)
        this.wallet = new FileSystemWallet(this.walletPath) // IWallet
    }

    async getWallet() { // Promise<IWallet>
        await this.initialized()

        return this.wallet
    }

    // affiliationsPromises = {} // {[key: string]: Promise<void>}

    // async checkOrCreateAffiliation(
    //     affiliation // string
    // ) { // Promise<void>
    //     this.initialized()
    //
    //     const normalizedAffiliation = affiliation.toLocaleLowerCase()
    //     if (!this.affiliationsPromises[normalizedAffiliation]) {
    //         this.affiliationsPromises[normalizedAffiliation] = (async() => {
    //             const adminIdentity = await this.setUserContext(CA_ADMIN_LABEL)
    //             const affiliationService = this.ca.newAffiliationService()
    //             const registeredAffiliationsResponse = await affiliationService.getAll(adminIdentity) // IRegisteredAffilationResponse;
    //             const registeredAffiliations = registeredAffiliationsResponse.result.affiliations || []
    //             const affiliationExists = registeredAffiliations.some((x) => {
    //                 const nameEquals = x.name.toLowerCase() === affiliation.toLowerCase()
    //                 return (
    //                     nameEquals
    //                     || (x.affiliations || []).some((y) => y.name.toLowerCase() === affiliation.toLowerCase())
    //                 )
    //             })
    //
    //             if (!affiliationExists) {
    //                 await affiliationService.create({name: affiliation, force: true}, adminIdentity)
    //             }
    //         })()
    //     }
    //
    //     return this.affiliationsPromises[normalizedAffiliation]
    // }

    // async createIdentity(
    //     label, // string
    //     affiliation = undefined // string
    // ) { // Promise<ICreatedIdentity | null>
    //     await this.initialized()
    //
    //     // Check to see if we've already enrolled the user.
    //     const userExists = await this.wallet.exists(label)
    //     if (userExists) {
    //         console.error(`An identity for the user "${label}" already exists in the wallet`)
    //         return null
    //     }
    //
    //     const adminIdentity = await this.setUserContext(CA_ADMIN_LABEL)
    //
    //     const fullAffiliation = affiliation ? `${this.orgConfig.mspid}.${affiliation}` : this.orgConfig.mspid
    //     await this.checkOrCreateAffiliation(fullAffiliation)
    //
    //     const secret = await this.ca.register({
    //         affiliation: fullAffiliation,
    //         enrollmentID: label,
    //         role: 'client'
    //     }, adminIdentity)
    //
    //     const enrollment = await this.ca.enroll({
    //         enrollmentID: label,
    //         enrollmentSecret: secret
    //     })
    //
    //     const userIdentity = X509WalletMixin.createIdentity(
    //         this.orgConfig.mspid,
    //         enrollment.certificate,
    //         enrollment.key.toBytes(),
    //     )
    //
    //     await this.wallet.import(label, userIdentity)
    //     // eslint-disable-next-line max-len
    //     console.log(`Successfully registered and enrolled user "${label}" and imported it into the ${this.orgName} wallet`)
    //
    //     return userIdentity // ICreatedIdentity
    // }

    // async setUserContext(
    //     label // string
    // ) { // Promise<FabricClient.User>
    //     // const gateway = new Gateway();
    //     // await gateway.connect(this.client, { wallet: this.wallet, identity: label, discovery: { enabled: false } });
    //     // return gateway.getCurrentIdentity();
    //     return this.wallet.setUserContext(this.client, label)
    // }

    // // private
    // async getPrivateKey(label: string): Promise<string> {
    //     const labelPath = path.resolve(this.walletPath, label)
    //     // eslint-disable-next-line max-len
    //     const meta = await JSON.parse(await readFile(path.resolve(labelPath, label), 'utf8')) // {enrollment: {signingIdentity: string}}
    //     const privateKeyPath = path.resolve(labelPath, `${meta.enrollment.signingIdentity}-priv`)
    //     return readFile(privateKeyPath, 'utf8')
    // }

    // private
    // initializePromise = undefined // Promise<void>

    async initialized() { // Promise<void>
        if (!this.initializePromise) {
            this.initializePromise = (async() => {
                await this.client.initCredentialStores()
                this.ca = this.client.getCertificateAuthority()

                this.orgConfig = networkConfig.organizations[this.orgName]
                if (!this.orgConfig) {
                    throw new Error(`Invalid organization: ${this.orgName} not found!`)
                }

                if (this.orgConfig.certificateAuthorities.length === 0) {
                    throw new Error(`Invalid organization: ${this.orgName} not found!`)
                }

                this.caConfig = networkConfig.certificateAuthorities[this.orgConfig.certificateAuthorities[0]]

                await this.initializeCaAdmin()
                await this.initializeAdmin()
            })()
        }

        return this.initializePromise
    }

    // initializeAdminPromise? // Promise<void>
    async initializeAdmin() { // Promise<void>
        if (!this.initializeAdminPromise) {
            this.initializeAdminPromise = (async() => {
                // Check to see if we've already enrolled the admin user.
                const adminExists = await this.wallet.exists(ADMIN_LABEL)
                if (adminExists) {
                    return
                }

                const adminPrivateKeyPath = this.orgConfig.adminPrivateKey.path
                const adminCertPath = this.orgConfig.signedCert.path

                const [adminCert, adminPrivateKey] = await Promise.all([
                    readFile(adminCertPath, 'utf8'),
                    readFile(adminPrivateKeyPath, 'utf8')
                ])

                const identity = X509WalletMixin.createIdentity(this.orgConfig.mspid, adminCert, adminPrivateKey)
                await this.wallet.import(ADMIN_LABEL, identity)
                console.log(`Successfully imported "${ADMIN_LABEL}" user  into the ${this.orgName} wallet`)
            })()
        }

        return this.initializeAdminPromise
    }

    // initializeCaAdminPromise = undefined // Promise<void>

    async initializeCaAdmin() { // Promise<void>
        if (!this.initializeCaAdminPromise) {
            this.initializeCaAdminPromise = (async() => {
                // Check to see if we've already enrolled the admin user.
                const adminExists = await this.wallet.exists(CA_ADMIN_LABEL)
                if (adminExists) {
                    return
                }

                // Enroll the admin user if he doesn't exist yet
                const registrar = this.caConfig.registrar[0]
                const enrollment = await this.ca.enroll({
                    enrollmentID: registrar.enrollId,
                    enrollmentSecret: registrar.enrollSecret
                })
                const identity = X509WalletMixin.createIdentity(
                    this.orgConfig.mspid,
                    enrollment.certificate,
                    enrollment.key.toBytes(),
                )
                await this.wallet.import(CA_ADMIN_LABEL, identity)
                // eslint-disable-next-line max-len
                console.log(`Successfully enrolled "${CA_ADMIN_LABEL}" user and imported it into the ${this.orgName} wallet`)
            })()
        }

        return this.initializeCaAdminPromise
    }
}

// export interface ICreatedIdentity extends Identity {
//     mspId: string;
//     certificate: string;
//     privateKey: string;
// }

// interface IWallet extends FileSystemWallet {
//     // @note setUserContext is a private API from the Wallet ..
//     //       but is better then the alternative way of setting up a Gateway
//     setUserContext(client: FabricClient, label: string): FabricClient.User;
// }

// interface IRegisteredAffilationResponse extends FabricCAServices.IServiceResponse {
//     result: {
//         affiliations?: {
//             name: string;
//             affiliations: {
//                 name: string;
//             }[];
//         }[];
//     };
// }

module.exports = {IdentityService}
