const { Transform } = require('@theledger/fabric-mock-stub')

const { proofIdRegex, proof } = require('../shapes')

let txId = 0

// function sample(array) {
//     return array[Math.floor ( Math.random() * array.length )]
// }

// function randomProofProps() {
//     return {
//         hospitalId: sample(['USTAN', 'ZMC', 'FCRB']),
//         date: new Date(),
//         status: sample(['success', 'failed']),
//     }
// }

class Proof {

    constructor() {
        this.proof = {
            ruleId:'RULE_123',
            serumsId: '123',
            request_received: {
                hospitalIds: ['USTAN'],
                date: new Date('2025-01-30').toISOString()
            }
        }

        this.create_request = {
            ruleId:'RULE_123',
            hospitalIds: ['USTAN'],
            serumsId: '123',
            date: new Date('2025-01-30').toISOString()
        }

        this.update_request = {
            proofId: 'PROOF_123',
            type: 'data_vault_created',
            hospitalId: 'USTAN',
            status: 'success',
            date: new Date('2025-01-30').toISOString()
        }
    }

    type_data_vault_created(hospitalId = 'USTAN') {
        const content = {
            dv_hash: "12341234",
            schema: "schema_1"
        }

        this.proof = {
            ...this.proof,
            data_vault_created: [
                {
                    hospitalId,
                    date: new Date('2025-01-30').toISOString(),
                    status: 'success',
                    ...content
                }
            ]
        }

        this.update_request = {
            ...this.update_request,
            hospitalId,
            type: 'data_vault_created',
            content
        }

        return this
    }

    type_data_selected(hospitalId = 'USTAN') {
        const content = {
            hash_TBD: "12341234"
        }

        this.proof = {
            ...this.proof,
            data_selected: [
                {
                    hospitalId,
                    date: new Date('2025-01-30').toISOString(),
                    status: 'success',
                    ...content
                }
            ]
        }

        this.update_request = {
            ...this.update_request,
            hospitalId,
            type: 'data_selected',
            content
        }

        return this
    }

    type_data_filled(hospitalId = 'USTAN') {
        const content = {
            hash_TBD: "12341234",
            error: "schema_1"
        }

        this.proof = {
            ...this.proof,
            data_filled: [
                {
                    hospitalId,
                    date: new Date('2025-01-30').toISOString(),
                    status: 'success',
                    ...content
                }
            ]
        }

        this.update_request = {
            ...this.update_request,
            hospitalId,
            type: 'data_filled',
            content
        }

        return this
    }

    type_encryption() {
        const content = {
            encryption_signature: "12341234"
        }

        this.proof = {
            ...this.proof,
            encryption: {
                date: new Date('2025-01-30').toISOString(),
                status: 'success',
                ...content
            }
        }

        delete this.update_request.hospitalId

        this.update_request = {
            ...this.update_request,
            type: 'encryption',
            content
        }

        return this
    }

    type_cleanup(hospitalId = 'USTAN') {
        const content = {
            dv_hash: "12341234",
            schema: "schema_1"
        }

        this.proof = {
            ...this.proof,
            cleanup: [
                {
                    hospitalId,
                    date: new Date('2025-01-30').toISOString(),
                    status: 'success',
                    ...content
                }
            ]
        }

        this.update_request = {
            ...this.update_request,
            hospitalId,
            type: "cleanup",
            content
        }

        return this
    }

    nextTxId() {
        txId += 1
        return `REQUEST_${txId}`
    }

    getFullProof() {
        return this.proof
    }

    async createProof(mockStub, expectSuccess = true) {
        const proofToCreate = {
            ruleId: this.create_request.ruleId,
            hospitalIds: this.create_request.hospitalIds,
            serumsId: this.create_request.serumsId,
            type: this.create_request.type,
            date: this.create_request.date,
        }

        const response = await mockStub.mockInvoke(
            this.nextTxId(),
            ['createProof', JSON.stringify(proofToCreate)]
        )

        if (expectSuccess) {
            expect(response.status).toBe(200)
            const payload = Transform.bufferToObject(response.payload)
            expect(payload.proofId).toEqual(expect.stringMatching(proofIdRegex))
            return payload.proofId
        }
        expect(response.status).toBe(500)
        const error = Transform.bufferToObject(response.message)
        return error
    }

    async updateProof(mockStub, proofId, expectSuccess = true) {
        const proofToUpdate = {
            id: proofId,
            type: this.update_request.type,
            hospitalId: this.update_request.hospitalId,
            status: this.update_request.status,
            content: this.update_request.content,
            date: this.update_request.date
        }

        const response = await mockStub.mockInvoke(
            this.nextTxId(),
            ['updateProof', JSON.stringify(proofToUpdate), proofId]
        )

        if (expectSuccess) {
            expect(response.status).toBe(200)
            const payload = Transform.bufferToObject(response.payload)
            expect(payload.proofId).toEqual(expect.stringMatching(proofIdRegex))
            return payload.proofId
        }
        expect(response.status).toBe(500)
        const error = Transform.bufferToObject(response.message)
        return error
    }
}

module.exports = Proof
