const {
    Transform
} = require("@theledger/fabric-mock-stub")

const getToken = require('./getToken')

let txId = 0

class PatientAccessRequest {

    /**
    *   Usage:
    *       new Request()
    *           .user('DOCTOR_1')
    *           .withGroups(['DOCTOR'])
    *           .from('HOSPITAL_1')
    *           .wantsToAccess(['BRAIN', 'CELL']) // or .wantsToAccess('all')
    *           .ofPatient('PATIENT_1')
    *           .from('HOSPITAL_1')
    */
    constructor() {
        this.request = {
            requestor: {},
            target: {}
        }
    }

    nextTxId() {
        txId += 1
        return `REQUEST_${txId}`
    }

    user(userId) {
        this.request.requestor.individualId = userId
        return this
    }

    withGroups(groups) {
        this.request.requestor.groupIds = groups
        return this
    }

    withDepartments(departments) {
        this.request.requestor.departmentIds = departments
        return this
    }

    from(orgId) {
        if (this.request.requestor.orgId) {
            this.request.target.patientOrgId = orgId
        } else {
            this.request.requestor.orgId = orgId
        }
        return this
    }

    wantsToAccess(attrs) {
        if (attrs === 'all') {
            // Nothing to do
            return this
        }
        this.request.target.access = attrs.map((att) => {
            return {name: att}
        })
        return this
    }

    ofPatient(userId) {
        this.request.target.patientId = userId
        return this
    }

    async makeRequest(mockStub, expectAccessGranted = true) {
        const token = getToken({
            id: this.request.requestor.individualId,
            orgId: this.request.requestor.orgId,
            groupIds: this.request.requestor.groupIds,
            departmentIds: this.request.requestor.departmentIds
        })

        const requestFunction = this.request.target.access ? 'getPatientAccess' : 'getPatientAccessAll'

        const response = await mockStub.mockInvoke(
            this.nextTxId(),
            [requestFunction, token, JSON.stringify(this.request.target)]
        )

        expect(response.status).toBe(200)
        const payload = Transform.bufferToObject(response.payload)
        expect(payload.accessGranted).toBe(expectAccessGranted)
        return payload
    }
}

module.exports = PatientAccessRequest
